package mvcaur;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import mvcaur.def.DefaultObjectFactory;

/**
 * Base class for the mvcaur routing configuration.
 * 
 * @author henper
 * 
 */
public abstract class Router {

	private List<RoutingFlow> flows = new ArrayList<RoutingFlow>();
	private ObjectFactory objectFactory = new DefaultObjectFactory();

	/**
	 * Inject an object factory
	 * 
	 * @param objectFactory
	 */
	public void setObjectFactory(ObjectFactory objectFactory) {
		this.objectFactory = objectFactory;
	}

	/**
	 * Finds a flow for a specific request URI.
	 * 
	 * @param requestURI
	 * @param requestParams
	 * @return
	 */
	public final PreparedFlow execute(String requestURI,
			Map<String, String[]> requestParams) {
		for (RoutingFlow flow : flows) {
			Controller<?> ctrl = flow.execute(requestURI, requestParams,
					objectFactory);

			if (ctrl != null) {
				PreparedFlow pflow = new PreparedFlow();
				pflow.setFlow(flow);
				pflow.setPreparedController(ctrl);
				return pflow;
			}
		}
		return null;
	}

	/**
	 * Execute without request params
	 * 
	 * @param uri
	 * @return
	 */
	public PreparedFlow execute(String uri) {
		return execute(uri, null);
	}

	/**
	 * This is where all url mappings are configured.
	 */
	public abstract void init();

	/**
	 * Start configuring routing for a specific url pattern.
	 * 
	 * @param route
	 * @return a routing flow, optimized for method chaining
	 */
	protected RoutingFlow route(String route) {
		RoutingFlow flow = new RoutingFlow();
		flow.route(route);
		flows.add(flow);
		return flow;
	}

}
